@@ -2,47 +2,25 @@
 
 from __future__ import print_function
 
+import itertools
 import json
 import os
 import os.path as osp
 import re
 import sys
 import textwrap
-from itertools import islice
 
-import requests
-from bs4 import BeautifulSoup
+import bs4
 
+from .download import _get_session
 from .download import download
 from .download import indent
 
-client = requests.session()
-
-folders_url = "https://drive.google.com/drive/folders/"
-files_url = "https://drive.google.com/uc?id="
-folder_type = "application/vnd.google-apps.folder"
-
-string_regex = re.compile(r"'((?:[^'\\]|\\.)*)'")
-
-
 MAX_NUMBER_FILES = 50
 
 
-class GoogleDriveFile(object):
-    """Represent Google Drive file objects structure.
-
-    Attributes
-    ----------
-    id: str
-        Unique id, used to build the download URL.
-    name: str
-        Actual name, used as file name.
-    type: str
-        MIME type, or application/vnd.google-apps.folder if it is a folder
-    children: List[GoogleDriveFile]
-        If it is a directory, it contains the folder files/directories
-
-    """
+class _GoogleDriveFile(object):
+    TYPE_FOLDER = "application/vnd.google-apps.folder"
 
     def __init__(self, id, name, type, children=None):
         self.id = id
@@ -51,40 +29,13 @@ class GoogleDriveFile(object):
         self.children = children if children is not None else []
 
     def is_folder(self):
-        return self.type == folder_type
-
-    def __repr__(self):
-        template = "(id={id}, name={name}, type={type}, children={children})"
-        return "GoogleDriveFile" + template.format(
-            id=self.id,
-            name=self.name,
-            type=self.type,
-            children=self.children,
-        )
+        return self.type == self.TYPE_FOLDER
 
 
-def parse_google_drive_file(folder, content, use_cookies=True):
-    """Extracts information about the current page file and its children
+def _parse_google_drive_file(folder, content):
+    """Extracts information about the current page file and its children."""
 
-    Parameters
-    ----------
-    folder: str
-        URL of the Google Drive folder.
-        Must be of the format 'https://drive.google.com/drive/folders/{url}'.
-    content: str
-        Google Drive's raw string
-
-    Returns
-    -------
-    gdrive_file: GoogleDriveFile
-        Current GoogleDriveFile, with empty children
-    id_name_type_iter: Iterator
-        Tuple iterator of each children id, name, type
-    """
-    folder_soup = BeautifulSoup(content, features="html.parser")
-
-    if not use_cookies:
-        client.cookies.clear()
+    folder_soup = bs4.BeautifulSoup(content, features="html.parser")
 
     # finds the script tag with window['_DRIVE_ivd']
     encoded_data = None
@@ -93,10 +44,14 @@ def parse_google_drive_file(folder, content, use_cookies=True):
 
         if "_DRIVE_ivd" in inner_html:
             # first js string is _DRIVE_ivd, the second one is the encoded arr
-            regex_iter = string_regex.finditer(inner_html)
+            regex_iter = re.compile(r"'((?:[^'\\]|\\.)*)'").finditer(
+                inner_html
+            )
             # get the second elem in the iter
             try:
-                encoded_data = next(islice(regex_iter, 1, None)).group(1)
+                encoded_data = next(
+                    itertools.islice(regex_iter, 1, None)
+                ).group(1)
             except StopIteration:
                 raise RuntimeError(
                     "Couldn't find the folder encoded JS string"
@@ -116,17 +71,21 @@ def parse_google_drive_file(folder, content, use_cookies=True):
 
     folder_contents = [] if folder_arr[0] is None else folder_arr[0]
 
-    seps = [" - ", " – "]  # unicode dash and endash
-    for sep in seps:
-        splitted = folder_soup.title.contents[0].split(sep)
-        if len(splitted) >= 2:
-            name = sep.join(splitted[:-1])
-            break
+    sep = " - "  # unicode dash
+    splitted = folder_soup.title.contents[0].split(sep)
+    if len(splitted) >= 2:
+        name = sep.join(splitted[:-1])
+    else:
+        raise RuntimeError(
+            "file/folder name cannot be extracted from: {}".format(
+                folder_soup.title.contents[0]
+            )
+        )
 
-    gdrive_file = GoogleDriveFile(
+    gdrive_file = _GoogleDriveFile(
         id=folder.split("/")[-1],
         name=name,
-        type=folder_type,
+        type=_GoogleDriveFile.TYPE_FOLDER,
     )
 
     id_name_type_iter = [
@@ -137,50 +96,35 @@ def parse_google_drive_file(folder, content, use_cookies=True):
     return gdrive_file, id_name_type_iter
 
 
-def download_and_parse_google_drive_link(
+def _download_and_parse_google_drive_link(
+    sess,
     folder,
     quiet=False,
-    use_cookies=True,
     remaining_ok=False,
+    verify=True,
 ):
-    """Get folder structure of Google Drive folder URL.
+    """Get folder structure of Google Drive folder URL."""
 
-    Parameters
-    ----------
-    folder: str
-        URL of the Google Drive folder.
-        Must be of the format 'https://drive.google.com/drive/folders/{url}'.
-    quiet: bool, optional
-        Suppress terminal output.
-    use_cookies: bool, optional
-        Flag to use cookies. Default is True.
-    remaining_ok: bool, optional
-        Flag that ensures that is ok to let some file to not be downloaded,
-        since there is a limitation of how many items gdown can download,
-        default is False.
-
-    Returns
-    -------
-    return_code: bool
-        Returns False if the download completed unsuccessfully.
-        May be due to invalid URLs, permission errors, rate limits, etc.
-    gdrive_file: GoogleDriveFile
-        Returns the folder structure of the Google Drive folder.
-    """
     return_code = True
 
-    folder_page = client.get(folder)
+    # canonicalize the language into English
+    if "?" in folder:
+        folder += "&hl=en"
+    else:
+        folder += "?hl=en"
+
+    folder_page = sess.get(folder, verify=verify)
 
     if folder_page.status_code != 200:
         return False, None
 
-    gdrive_file, id_name_type_iter = parse_google_drive_file(
+    gdrive_file, id_name_type_iter = _parse_google_drive_file(
         folder,
         folder_page.text,
     )
 
     for child_id, child_name, child_type in id_name_type_iter:
-        if child_type != folder_type:
+        if child_type != _GoogleDriveFile.TYPE_FOLDER:
             if not quiet:
                 print(
                     "Processing file",
@@ -188,7 +132,7 @@ def download_and_parse_google_drive_link(
                     child_name,
                 )
             gdrive_file.children.append(
-                GoogleDriveFile(
+                _GoogleDriveFile(
                     id=child_id,
                     name=child_name,
                     type=child_type,
@@ -204,9 +148,9 @@ def download_and_parse_google_drive_link(
                 child_id,
                 child_name,
             )
-        return_code, child = download_and_parse_google_drive_link(
-            folders_url + child_id,
-            use_cookies=use_cookies,
+        return_code, child = _download_and_parse_google_drive_link(
+            sess,
+            "https://drive.google.com/drive/folders/" + child_id,
             quiet=quiet,
             remaining_ok=remaining_ok,
         )
@@ -218,7 +162,7 @@ def download_and_parse_google_drive_link(
         err_msg = " ".join(
             [
                 "The gdrive folder with url: {url}".format(url=folder),
-                "has at least {max} files,".format(max=MAX_NUMBER_FILES),
+                "has more than {max} files,".format(max=MAX_NUMBER_FILES),
                 "gdrive can't download more than this limit,",
                 "if you are ok with this,",
                 "please run again with --remaining-ok flag.",
@@ -228,21 +172,9 @@ def download_and_parse_google_drive_link(
     return return_code, gdrive_file
 
 
-def get_directory_structure(gdrive_file, previous_path):
-    """Converts a Google Drive folder structure into a local directory list.
+def _get_directory_structure(gdrive_file, previous_path):
+    """Converts a Google Drive folder structure into a local directory list."""
 
-    Parameters
-    ----------
-    gdrive_file: GoogleDriveFile
-        Google Drive folder structure.
-    previous_path: str
-        Path containing the parent's file path.
-
-    Returns
-    -------
-    directory_structure: list
-        List containing a tuple of the files' ID and file path.
-    """
     directory_structure = []
     for file in gdrive_file.children:
         file.name = file.name.replace(osp.sep, "_")
@@ -250,7 +182,7 @@ def get_directory_structure(gdrive_file, previous_path):
             directory_structure.append(
                 (None, osp.join(previous_path, file.name))
             )
-            for i in get_directory_structure(
+            for i in _get_directory_structure(
                 file, osp.join(previous_path, file.name)
             ):
                 directory_structure.append(i)
@@ -270,6 +202,7 @@ def download_folder(
     speed=None,
     use_cookies=True,
     remaining_ok=False,
+    verify=True,
 ):
     """Downloads entire folder from URL.
 
@@ -291,6 +224,10 @@ def download_folder(
         Download byte size per second (e.g., 256KB/s = 256 * 1024).
     use_cookies: bool, optional
         Flag to use cookies. Default is True.
+    verify: bool or string
+        Either a bool, in which case it controls whether the server's TLS
+        certificate is verified, or a string, in which case it must be a path
+        to a CA bundle to use. Default is True.
 
     Returns
     -------
@@ -302,7 +239,6 @@ def download_folder(
     gdown.download_folder(
         "https://drive.google.com/drive/folders/" +
         "1ZXEhzbLRLU1giKKRJkjm8N04cO_JoYE2",
-        use_cookies=True
     )
     """
     if not (id is None) ^ (url is None):
@@ -310,14 +246,17 @@ def download_folder(
     if id is not None:
         url = "https://drive.google.com/drive/folders/{id}".format(id=id)
 
+    sess = _get_session(use_cookies=use_cookies)
+
     if not quiet:
         print("Retrieving folder list", file=sys.stderr)
     try:
-        return_code, gdrive_file = download_and_parse_google_drive_link(
+        return_code, gdrive_file = _download_and_parse_google_drive_link(
+            sess,
             url,
             quiet=quiet,
-            use_cookies=use_cookies,
             remaining_ok=remaining_ok,
+            verify=verify,
         )
     except RuntimeError as e:
         print("Failed to retrieve folder contents:", file=sys.stderr)
@@ -337,7 +276,7 @@ def download_folder(
         root_folder = osp.join(output, gdrive_file.name)
     else:
         root_folder = output
-    directory_structure = get_directory_structure(gdrive_file, root_folder)
+    directory_structure = _get_directory_structure(gdrive_file, root_folder)
     if not osp.exists(root_folder):
         os.makedirs(root_folder)
 
@@ -351,12 +290,13 @@ def download_folder(
             continue
 
         filename = download(
-            files_url + file_id,
+            "https://drive.google.com/uc?id=" + file_id,
             output=file_path,
             quiet=quiet,
             proxy=proxy,
             speed=speed,
             use_cookies=use_cookies,
+            verify=verify,
         )
 
         if filename is None:
